info = {
    'icon': 'MOD_CLOTH',
    'description': 'Decimate cloth',
}

import bpy
from pathlib import Path
import re


def set_panel(panel):
    '''take a panel name and apply it to properties zone'''
    for area in bpy.context.screen.areas:
        if area.type == 'PROPERTIES':
            for space in area.spaces:
                if space.type == 'PROPERTIES':
                    space.context = panel
                    return (1)
    return (0)


def get_panel():
    '''return active panel name of the properties zone'''
    for area in bpy.context.screen.areas:
        if area.type == 'PROPERTIES':
            for space in area.spaces:
                if space.type == 'PROPERTIES':
                    return(space.context)
    return (0)

def get_override():
    for window in bpy.context.window_manager.windows:
        screen = window.screen
        for area in screen.areas:
            if area.type == 'PROPERTIES':
                #for region in area.regions:
                #    if region.type == 'WINDOW':
                return {'window': window, 'screen': screen, 'area': area}


def simplify_cloth(poly_target):
    if not get_override():
        print('ERROR', f'Need a properties windows editor open !')
        return

    obj = bpy.context.object

    mods = obj.modifiers

    polycount = len(obj.data.polygons)

    if polycount < poly_target:
        print('ERROR', f'Already lowpoly ({polycount} polygons)')
        return

    target_ratio = poly_target / polycount
    print('target_ratio: ', target_ratio)

    sdef = None

    for m in mods:
        if m.type == 'SURFACE_DEFORM':
            sdef = m
            break

    if not sdef:
        print('ERROR', 'no surface deform found')
        return

    set_panel('MODIFIER')
    properties_override = get_override()

    if sdef.is_bound:
        bpy.ops.object.surfacedeform_bind(properties_override, modifier="SurfaceDeform")

    # create decimate and move to top
    decim = obj.modifiers.new('Decimate', 'DECIMATE')
    bpy.ops.object.modifier_move_to_index(modifier="Decimate", index=0)

    # use target ratio
    decim.ratio = target_ratio
    # apply 
    bpy.ops.object.modifier_apply(modifier="Decimate")

    # rebind
    bpy.ops.object.surfacedeform_bind(properties_override, modifier="SurfaceDeform")



simplify_cloth(15000)