info = {
    'icon' : 'ACTION_TWEAK',
    'description' : 'Bake action of selected objects',
    'selection' : True
}

import bpy
from os.path import abspath, relpath, dirname, basename, join
from time import time
C = bpy.context
D = bpy.data
scene = C.scene

def set_collection(ob, collection, unlink=True) :
    ''' link an object in a collection and create it if necessary, if unlink object is removed from other collections'''
    scn     = bpy.context.scene
    col     = None
    visible = False
    linked  = False

    # check if collection exist or create it
    for c in bpy.data.collections :
        if c.name == collection : col = c
    if not col : col = bpy.data.collections.new(name=collection)

    # link the collection to the scene's collection if necessary
    for c in scn.collection.children :
        if c.name == col.name : visible = True
    if not visible : scn.collection.children.link(col)

    # check if the object is already in the collection and link it if necessary
    for o in col.objects :
        if o == ob : linked = True
    if not linked : col.objects.link(ob)

    # remove object from scene's collection
    for o in scn.collection.objects :
        if o == ob : scn.collection.objects.unlink(ob)

    # if unlink flag we remove the object from other collections
    if unlink :
        for c in ob.users_collection :
            if c.name != collection : c.objects.unlink(ob)



def set_soft_body(ob, mod, bake=False):
    if bake:
        print(f'baking {ob.name} > {mod.name}')
    ptc = mod.point_cache

    name = ptc.name
    override = {'scene': bpy.context.scene,
    'active_object': ob,#no sure of this line.. .
    'point_cache': ptc}
    if bake:
        if name:#kill point cache related to point cache name
            bpy.ops.ptcache.free_bake(override)
            ## bpy.ops.ptcache.free_bake_all(override)

            #ptc.is_outdated = True # read-only

    ptc.frame_start = scene.frame_start
    ptc.frame_end = scene.frame_end

    if not ptc.use_disk_cache:
        ptc.use_disk_cache = True
        print('enabling use disk cache')
    
    if bake:
        print('>>> baking')
        start = time()
        #bpy.ops.ptcache.bake(override, bake=False)# bake to current frame
        bpy.ops.ptcache.bake(override, bake=True)
        print(f'Done {time()-start:.3f}') 

def set_particle_system(ob, mod, bake=False):
    if bake:
        print(f'baking {ob.name} > {mod.name}')
    ptc = mod.particle_system.point_cache
    ptc.frame_start = scene.frame_start
    ptc.frame_end = scene.frame_end
    # ptc.filepath = join(dirname(D.filepath), splitext(basename)[0]+'_cache')
    if not ptc.use_disk_cache:
        ptc.use_disk_cache = True
        print('enabling use disk cache')
    
    if bake:
        override = {'scene': bpy.context.scene,
        'active_object': ob,#no sure of this line.. .
        'point_cache': ptc}
        print('>>> baking')
        start = time()
        #bpy.ops.ptcache.bake(override, bake=False)# bake to current frame
        bpy.ops.ptcache.bake(override, bake=True)
        print(f'Done {time()-start:.3f}')

def bake_mods(ob, bake=False):
    for mod in ob.modifiers:
        if mod.type == 'SOFT_BODY':
            set_soft_body(ob, mod, bake)
        
        if mod.type == 'PARTICLE_SYSTEM':
            set_particle_system(ob, mod, bake)



def bake_action(selected=True):
    if selected:
        pool = []
        for o in C.selected_objects:
            if o.animation_data.action:
                print(f'{o.name} already has an action')
                o.select_set(False)
                continue
            else:
                pool.append(o)
        if not pool:
            print('no object ready for action bake')
            return
    
    else:#go in physics ob collection
        phycol = D.collections.get('Physics_sim')
        if not phycol:
            print('No physics collection')
            return

        pool = [o for o in phycol.all_objects if not o.hide_viewport and not o.animation_data.action]
        #redefine selection 
        bpy.ops.object.select_all(action='DESELECT')
        for o in pool:
            o.select_set(True)
            C.view_layer.objects.active = o

    # get viewport override ? (no need if launched  from custom shelf)

    print('Baking action...')
    # launch action baking
    bpy.ops.nla.bake(frame_start=scene.frame_start, frame_end=scene.frame_end, only_selected=False, visual_keying=True, clear_constraints=True, clear_parents=True, bake_types={'OBJECT'})


bake_action(selected=info['selection'])
print('EOF')