info = {
    'icon' : 'PHYSICS',
    'description' : 'Bake modifier cache of selected objects',
    'selection' : True
}

import bpy
from os.path import abspath, relpath, dirname, basename, join
from time import time
C = bpy.context
D = bpy.data
scene = C.scene


def set_soft_body(ob, mod, bake=False):
    if bake:
        print(f'baking {ob.name} > {mod.name}')
    ptc = mod.point_cache

    name = ptc.name
    override = {'scene': bpy.context.scene,
    'active_object': ob,#no sure of this line.. .
    'point_cache': ptc}
    if bake:
        if name:#kill point cache related to point cache name
            bpy.ops.ptcache.free_bake(override)
            ## bpy.ops.ptcache.free_bake_all(override)

            #ptc.is_outdated = True # read-only

    ptc.frame_start = scene.frame_start
    ptc.frame_end = scene.frame_end

    if not ptc.use_disk_cache:
        ptc.use_disk_cache = True
        print('enabling use disk cache')
    
    if bake:
        print('>>> baking')
        start = time()
        #bpy.ops.ptcache.bake(override, bake=False)# bake to current frame
        bpy.ops.ptcache.bake(override, bake=True)
        print(f'Done {time()-start:.3f}') 

def set_particle_system(ob, mod, bake=False):
    if bake:
        print(f'baking {ob.name} > {mod.name}')
    ptc = mod.particle_system.point_cache
    ptc.frame_start = scene.frame_start
    ptc.frame_end = scene.frame_end
    # ptc.filepath = join(dirname(D.filepath), splitext(basename)[0]+'_cache')
    if not ptc.use_disk_cache:
        ptc.use_disk_cache = True
        print('enabling use disk cache')
    
    if bake:
        override = {'scene': bpy.context.scene,
        'active_object': ob,#no sure of this line.. .
        'point_cache': ptc}
        print('>>> baking')
        start = time()
        #bpy.ops.ptcache.bake(override, bake=False)# bake to current frame
        bpy.ops.ptcache.bake(override, bake=True)
        print(f'Done {time()-start:.3f}')

def bake_mods(ob, bake=False):
    for mod in ob.modifiers:
        if mod.type == 'SOFT_BODY':
            set_soft_body(ob, mod, bake)
        
        if mod.type == 'PARTICLE_SYSTEM':
            set_particle_system(ob, mod, bake)


def bake_mod_cache(selected=True):
    phycol = D.collections.get('Physics_sim')
    if not phycol:
        print('No physics collection')
        return

    if selected:
        pool = []
        for o in C.selected_objects:
            if not phycol in o.users_collection:
                print(f'{o.name} is not in Physics_sim collection' )
                continue
            pool.append(o)

        if not pool:
            print('no object ready for cache baking')
            return
    
    else:#go in physics ob collection
        pool = [o for o in phycol.all_objects if not o.hide_viewport and not o.animation_data.action]
    #automate preroll ?
    for o in pool:
        bake_mods(o, bake=True)

bake_mod_cache(selected=info['selection'])
print('EOF')